; Proof of concept code for reading/writing ds1315
; rtc in Dragon rom socket
;
; This is for the specific case where the ds1315 is
; co-resident with the rom that contains the cpu vectors
; with no means of deselecting the rom.
;
; Should allow use of Mike Miller's ds1315 rtc board
; in a Dragon 32 with a suitable adaptor.
;
; The code is fairly small but it requires a 428 byte
; buffer to compile the instruction sequences
;
; Refer to Mike's documentation for further details.
;
; S.Orchard Dec 2021


RTCBASE equ $a000


    org $4000

    bra read_test

write_test

    ldu #inst_buf
    ldx #accesskey
    bsr build_writer
    ldx #rtcdata
    bsr build_writer
    bra run_routine

read_test

    ldu #inst_buf
    ldx #accesskey
    bsr build_writer
    ldx #$400       ; show raw rtc data on text screen
    bsr build_reader


run_routine
    pshs cc
    orcc #$50

    lda #$39        ; terminate buffer with rts
    sta ,u          ;

    ldx #RTCBASE    ; address for rtc write 0
    leay 1,x        ; address for rtc write 1
    jsr inst_buf

    puls cc,pc

;--------------------------------------
; build a 64-bit write routine
;
;   x : points to 8 byte sequence to write
;   u : points to instruction buffer
; returns
;   u : points to byte after last instruction
;
; compiled sequence consists purely
; of lda ,x and lda ,y instructions
; (128 bytes)
;
; instructions must not contain nvma cycles
; as these would break the write sequence
;
build_writer
    ldy #8
w1  lda ,x+

    ldb #8
    std temp    ; byte in temp, 8 in count
    lda #$a6    ; lda indexed
w2  ldb #$84    ; lda ,x  (write 0)
    lsr temp
    bcc w3
    ldb #$a4    ; lda ,y  (write 1)
w3  std ,u++
    dec count
    bne w2

    leay -1,y
    bne w1
    rts

;--------------------------------------
; build a 64-bit read routine
;
;   x : points to 8 byte receive buffer
;   u : points to instruction buffer
; returns
;   u : points to byte after last instruction
;
; compiled sequence consists of:
;   ldx #RTCBASE+4    (to set up rtc read address)
; for each byte n 0..7:
;     for each bit 0..7:
;         lda ,x
;         lsra
;         rora
;     ldu #rxbuf+n
;     stb ,u
; (299 bytes)
;
; instructions must not contain nvma cycles
; as these would break the read sequence
;
build_reader
    lda #8          ; 8 bytes
    sta count

    lda #$8e        ; ldx #
    sta ,u+
    ldd #RTCBASE+4
    std ,u++

r1  ldy #8          ; 8 bits
    pshs x
    ldd #$a684      ; lda ,x
    ldx #$4456      ; lsra rorb
r2  std ,u++
    stx ,u++
    leay -1,y
    bne r2
    puls x

    lda #$ce        ; ldu #
    sta ,u+
    stx ,u++        ; address where we want to store b
    ldd #$e7c4      ; stb ,u
    std ,u++

    leax 1,x        ; inc store address for next byte
    dec count
    bne r1
    rts

accesskey   fcb $c5,$3a,$a3,$5c,$c5,$3a,$a3,$5c
rtcdata     fcb 0,0,0,0,$14,$02,$12,$21

temp    rmb 1
count   rmb 1

inst_buf
        rmb 128     ; access key writer
        rmb 300     ; reader or writer + rts

endbuf  equ *
